import discord
from discord.ext import commands
from openai import OpenAI
import os
from dotenv import load_dotenv

# Load environment variables
load_dotenv()

# Configure OpenAI client to point to OpenWebUI
client = OpenAI(
    api_key=os.getenv('OPENAI_API_KEY'),
    base_url=os.getenv('OPENWEBUI_API_BASE')  # e.g., "http://localhost:8080/v1"
)

# Initialize Discord bot
intents = discord.Intents.default()
intents.message_content = True
bot = commands.Bot(command_prefix="!", intents=intents)

async def get_chat_history(channel, limit=50):
    messages = []
    async for message in channel.history(limit=limit):
        # Skip bot's own messages
        if message.author == bot.user:
            continue
        messages.append({"role": "user", "content": message.content})
    return list(reversed(messages))  # Return in chronological order

async def get_ai_response(messages):
    try:
        response = client.chat.completions.create(
            model=os.getenv('MODEL_NAME') or "us.anthropic.claude-3-5-sonnet-20241022-v2:0",
            messages=messages,
            temperature=0.7,
            max_tokens=500
        )
        return response.choices[0].message.content
    except Exception as e:
        print(f"Error getting AI response: {e}")
        return "Sorry, I encountered an error while processing your request."

@bot.event 
async def on_message(message):
    # Ignore messages from the bot itself
    if message.author == bot.user:
        return

    # Respond to DMs or when mentioned in a server
    if isinstance(message.channel, discord.DMChannel) or bot.user in message.mentions:
        # For mentions, remove the bot mention from the message
        if bot.user in message.mentions:
            prompt = message.content.replace(f'<@{bot.user.id}>', '').strip()
        else:
            prompt = message.content.strip()
        
        # If there's no prompt
        if not prompt:
            await message.channel.send("Hello! How can I help you?")
            return

        # Show typing indicator while processing
        async with message.channel.typing():
            # Get chat history
            chat_history = await get_chat_history(message.channel)
            
            # Add current message to history
            chat_history.append({"role": "user", "content": prompt})
            
            # Get response from OpenWebUI with chat history context
            response = await get_ai_response(chat_history)
            
            # Send the response
            # Split long messages if they exceed Discord's character limit
            if len(response) > 2000:
                chunks = [response[i:i+2000] for i in range(0, len(response), 2000)]
                for chunk in chunks:
                    await message.channel.send(chunk)
            else:
                await message.channel.send(response)

    await bot.process_commands(message)

def main():
    # Get the Discord token from environment variables
    discord_token = os.getenv('DISCORD_TOKEN')
    if not discord_token:
        raise ValueError("Discord token not found in environment variables")

    # Run the bot
    bot.run(discord_token)

if __name__ == "__main__":
    main()